//********************************************************************
UNIT SkyDome;
//********************************************************************
INTERFACE

USES dglOpenGl,glBitmap;
//********************************************************************
CONST
   SKYCOLS = 12; // (360/SKYCOLS) should be a an integer
   SKYROWS = 9;  // (90/SKYROWS) should be a an integer
   PIDIV180 = Pi/180;

TYPE
   TVertex3D = Record U,V,X,Y,Z: Single; End;
   TVertexArray = Array [0..SKYCOLS,0..SKYROWS-1] Of TVertex3D;

   TSkyDome = Class(TObject)
   Private
      TopVertex: TVertex3D;
      Vertices: TVertexArray;
      Texture: TglBitmap2D;
   Public
      Constructor Create(ASize: Integer;
                         ATextureName: String); Reintroduce;
      Destructor Destroy; Override;
      Procedure Render;
   End;

//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
Constructor TSkyDome.Create(ASize: Integer; ATextureName: String);
Var CylCos,CylSin,FactorLinear,FactorTrigon: Single;
    AngleCol,AngleRow,I,J: Integer;
Begin
   Inherited Create;

   With TopVertex Do Begin
      U := 0.5;
      V := 0.5;
      X := 0;
      Y := ASize;
      Z := 0;
   End;

   // Create Vertices
   AngleRow:= 0;
   For I:= 0 To SKYROWS-1 Do Begin

      AngleCol:= 0;
      For J:= 0 To SKYCOLS Do Begin  // Important! NOT (SKYCOLS-1)!
         With Vertices[J,I] Do Begin

            // Linear shortening factor for texture
            FactorLinear:= 1-AngleRow/90;
            // Trigonometric shortening factor for sphere point
            FactorTrigon:= Cos(AngleRow*PIDIV180);
            // Cylindric coordinates
            CylSin:= Sin(AngleCol*PIDIV180);
            CylCos:= Cos(AngleCol*PIDIV180);

            // Create X/Y/Z with trigonometric shortening factor
            X := CylCos * FactorTrigon * ASize;
            Z := CylSin * FactorTrigon * ASize;  // Attention! Sin = clockwise!
            Y := Sin(AngleRow*PIDIV180) * ASize;

            // Create U/V with linear shortening factor
            // (Using the trigonometric factor would create an orthogonal
            // projection of the vertices to the X/Z-Plane; this linear
            // method produces less distortion)
            U := (CylCos * FactorLinear) / 2 + 0.5;
            V := (CylSin * FactorLinear) / 2 + 0.5;

         End;
         Inc(AngleCol,360 Div SKYCOLS);
      End;

      Inc(AngleRow,90 Div SKYROWS);
   End;

   Texture:= TglBitmap2D.Create;
   Texture.LoadFromFile(ATextureName);
   Texture.GenTexture;
End;
//********************************************************************
Destructor TSkyDome.Destroy;
Begin
   Texture.Free;
   Inherited Destroy;
End;
//********************************************************************
Procedure TSkyDome.Render;
Var I,J: Integer;
Begin
   Texture.Bind;

   glColor4f(1,1,1,1);

   // Render body
   glBegin(GL_QUADS);
      For I:= 0 To SKYROWS-2 Do Begin
         For J:= 0 To SKYCOLS-1 Do Begin

            With Vertices[J,I] Do Begin
               glTexCoord2f(U,V);
               glNormal3f(-X,-Y,-Z);
               glVertex3f(X,Y,Z);
            End;
            With Vertices[J+1,I] Do Begin
               glTexCoord2f(U,V);
               glNormal3f(-X,-Y,-Z);
               glVertex3f(X,Y,Z);
            End;
            With Vertices[J+1,I+1] Do Begin
               glTexCoord2f(U,V);
               glNormal3f(-X,-Y,-Z);
               glVertex3f(X,Y,Z);
            End;
            With Vertices[J,I+1] Do Begin
               glTexCoord2f(U,V);
               glNormal3f(-X,-Y,-Z);
               glVertex3f(X,Y,Z);
            End;
         End;
      End;
   glEnd;

   // Render top plate
   glBegin(GL_TRIANGLES);
      For J:= 0 To SKYCOLS-1 Do Begin
         With Vertices[J,SKYROWS-1] Do Begin
            glTexCoord2f(U,V);
            glNormal3f(-X,-Y,-Z);
            glVertex3f(X,Y,Z);
         End;
         With Vertices[J+1,SKYROWS-1] Do Begin
            glTexCoord2f(U,V);
            glNormal3f(-X,-Y,-Z);
            glVertex3f(X,Y,Z);
         End;
         With TopVertex Do Begin
            glTexCoord2f(U,V);
            glNormal3f(-X,-Y,-Z);
            glVertex3f(X,Y,Z);
         End;
      End;
   glEnd;

   Texture.Unbind;
End;
//********************************************************************
//********************************************************************
End.
