//********************************************************************
UNIT TestFormUnit;
//********************************************************************

INTERFACE

USES
   Windows, Types, SysUtils,Classes, Controls, ExtCtrls, StdCtrls,
   Forms, Dialogs, Math, dglOpenGl, glBitmap, SkyDome;

//********************************************************************
Type
   TTestForm = Class(TForm)
      StartBtn: TButton;
      OGLPanel: TPanel;
      Timer1: TTimer;
      Procedure FormCreate(Sender: TObject);
      Procedure FormDestroy(Sender: TObject);
      Procedure FormResize(Sender: TObject);
      Procedure StartBtnClick(Sender: TObject);
      Procedure Timer1Timer(Sender: TObject);
      Procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
         MousePos: TPoint; var Handled: Boolean);
      Procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
         MousePos: TPoint; var Handled: Boolean);
   Private
      { Private-Deklarationen }
      Procedure RenderByOpenGl;
   Public
      { Public-Deklarationen }
   End;

VAR
   TestForm: TTestForm;
//********************************************************************
IMPLEMENTATION
//********************************************************************

{$R *.dfm}

//********************************************************************
VAR
      OGLDeviceContext: HDC;
      OGLRenderingContext: HGLRC;
      OGLWindowHandle: HWND;          

      MySkyDome: TSkyDome;
      RotAngle,YPos: Integer;
      LookUp: Boolean;
//********************************************************************
Procedure TTestForm.FormCreate(Sender: TObject);
Const
   Light_Position: Array[0..3] Of GlFloat = (-10,+10,+10,0);
   Light_Ambient : Array[0..3] Of GlFloat = (0.8, 0.8, 0.8, 1.0);
   Mat_Ambient   : Array[0..3] Of GlFloat = (0.8, 0.8, 0.8, 1.0);
Begin
   OGLWindowHandle:= OGLPanel.Handle;
   OGLDeviceContext:= GetDC(OGLWindowHandle);
   If Not InitOpenGL Then;

   OGLRenderingContext:= CreateRenderingContext
           (OGLDeviceContext,[opDoubleBuffered],32,24,0,0,0,0);
   ActivateRenderingContext(OGLDeviceContext, OGLRenderingContext);
   FormResize(Self);

   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);

   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

   glEnable(GL_Texture_2D);

   glDisable(GL_BLEND);     

   glPolygonMode(GL_FRONT,GL_FILL);

   glShadeModel(GL_SMOOTH);

   glEnable(GL_LIGHTING);
   glLightfv(GL_LIGHT0,GL_POSITION,@Light_Position);
   glLightfv(GL_LIGHT0,GL_AMBIENT,@Light_Ambient);
   glEnable(GL_LIGHT0);
   glEnable(GL_COLOR_MATERIAL);
   glMaterialfv(GL_FRONT, GL_AMBIENT,@Mat_Ambient);
   
   glClearColor(0,0,0,1);

   MySkyDome:= TSkyDome.Create(400,'Figures64X64.bmp');
   RotAngle:=0;
   YPos:= -200;
   LookUp:= FALSE;
End;
//********************************************************************
Procedure TTestForm.FormDestroy(Sender: TObject);
Begin
   MySkyDome.Free;

   DeactivateRenderingContext;
   DestroyRenderingContext(OGLRenderingContext);
   ReleaseDC(OGLWindowHandle, OGLDeviceContext);
End;
//********************************************************************
Procedure TTestForm.FormResize(Sender: TObject);
Begin
   OGLPanel.Width:= ClientWidth;
   OGLPanel.Height:= ClientHeight;
   With OGLPanel Do glViewport(0,0,Width,Height);
End;
//********************************************************************
Procedure TTestForm.StartBtnClick(Sender: TObject);
Begin
   LookUp:= Not LookUp;
End;
//********************************************************************
Procedure TTestForm.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
Begin
   Dec(YPos,5);
End;
//********************************************************************
Procedure TTestForm.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
Begin
   Inc(YPos,5);
End;
//********************************************************************
Procedure TTestForm.Timer1Timer(Sender: TObject);
Begin
   RenderByOpenGl;
End;
//********************************************************************
Procedure TTestForm.RenderByOpenGl;
Type TProjectionType = (ptOrtho,ptFrustum);
Var Viewport: Array [0..3] Of Integer;
    XMax,YMax,ZNear,ZFar: Single;
Begin
   glClear(GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   glGetIntegerv(GL_VIEWPORT, @Viewport);

   ZNear:= +1;  // Distance to near clipping plane, must be positive
   ZFar := 400;  // Distance to far clipping plane, must be positive
   YMax:= Tan(45/2*PI/180)*ZNear;
   XMax:= YMax*ViewPort[2]/ViewPort[3];
   glFrustum(-XMax,+XMax,-YMax,+YMax,ZNear,ZFar);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;
   If LookUp
      Then glRotatef(-90,1,0,0); // Look upwards
   glRotatef(RotAngle,0,1,0); // Rotate SkyDome
   Inc(RotAngle);
   glTranslatef(0,YPos,0);

   MySkyDome.Render;

   SwapBuffers(OGLDeviceContext);

   Caption:= gluErrorString(glGetError);
End;
//********************************************************************
//********************************************************************
End.

